﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Selection;
using System.Linq;

namespace HeuristicLab.Analysis.FitnessLandscape {
  [Item("Adaptive Walk", "An adaptive walk applies a certain manipulation operation always taking the best of the sample.")]
  [StorableClass]
  [Creatable(CreatableAttribute.Categories.TestingAndAnalysis + CreatableAttribute.Categories.SplitToken + "1" + CreatableAttribute.Categories.OrderToken + "FLA", Priority = 303)]
  public sealed class AdaptiveWalk : LocalAnalysis<BestSelector> {

    public IFixedValueParameter<IntValue> SampleSizeParameter {
      get { return (IFixedValueParameter<IntValue>)Parameters["SampleSize"]; }
    }

    [StorableConstructor]
    private AdaptiveWalk(bool deserializing) : base(deserializing) { }
    private AdaptiveWalk(AdaptiveWalk original, Cloner cloner) : base(original, cloner) { }
    public AdaptiveWalk()
      : base(new BestSelector()) {
      SelectorParameter.Value.NumberOfSelectedSubScopesParameter.Hidden = true;
      Parameters.Add(new FixedValueParameter<IntValue>("SampleSize", "Number of sampled moves from which the adaptive walk chooses the best.", new IntValue(10)));
      var trailAnalyzer = AnalyzerParameter.Value.Operators.OfType<QualityTrailMultiAnalyzer>().Single();
      var upDown = trailAnalyzer.Operators.OfType<UpDownWalkAnalyzer>().Single();
      trailAnalyzer.Operators.SetItemCheckedState(upDown, false);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AdaptiveWalk(this, cloner);
    }
  }
}
